!****h* TrasfMatrx/DerivTrasf_class
! NAME
! DerivTrasf_class -- 
! FUNCTION
! 
! AUTHOR
! Paolo Zoccarato
! CREATION DATE
! 2006/06/20
! MODIFICATION HISTORY
! +------------------------------------------------------------------+
! |  name      | date   |                  reason                    |
! |------------|--------|--------------------------------------------|
! |            |        |  No one at the moment                      |
! +------------|--------|--------------------------------------------+
! |            |        |                                            |
! +------------------------------------------------------------------+
! COMMANDS
! PUBLIC FUNCTIONS:
!
! USES
! This module uses:
! the functions "one, deg2rad" of the module "Globals_class"
! NOTES
! Develloped with Compaq Visual Fortran for Windows on Windows XP
! SOURCE

MODULE DerivTrasf_class

USE Globals_class, ONLY : deg2rad
USE RotoTrasl_class, ONLY : rot

! Modulo che contiene le derivate parziali delle matrici di trasformazione
!
! PUBLIC FUNCTIONS:

IMPLICIT NONE

PRIVATE :: dRlx, dRty, d2Rlx, d2Rty

CONTAINS

! ============================================================
! Derivate prime
! ============================================================

! ============================================================
FUNCTION dTPS(lambda, theta, l_i, lt)

! Calcolo della derivata della matrice di trasformazione TPS rispetto
! a A, omega, phi, dlambda_0, lambda_0, dtheta_0, theta_0
!
! PARAMETRI DI INPUT
! lambda : angolo dell'ampiezza delle precessione [deg]
! theta  : angolo della rotazione della precessione [deg]
! l_i    :
! lt     :
!
! PARAMETRI DI OUTPUT
! dTPS   : derivata della matrice di trasformazione TPS

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: lambda, theta, l_i
LOGICAL, INTENT(IN) :: lt
REAL(8) :: dTPS(4,4)

! Inizio procedimento
dTPS(1:3,1:3) = dRPS(lambda, theta, l_i, lt)
dTPS(1,4) = 0.0d0
dTPS(2,4) = 0.0d0
dTPS(3,4) = 0.0d0
dTPS(4,1) = 0.0d0
dTPS(4,2) = 0.0d0
dTPS(4,3) = 0.0d0
dTPS(4,4) = 0.0d0

END FUNCTION dTPS
! ============================================================

! ============================================================
FUNCTION dTBP(psi, l_i)

! Calcolo della derivata della matrice di trasformazione TBP rispetto a
! dpsi_0 e psi_0
!
! PARAMETRI DI INPUT
! l_i  :
! psi  : angolo di spin [deg]
!
! PARAMETRI DI OUTPUT
! dTBP : derivata della matrice di trasformazione TBP

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: psi, l_i
REAL(8) :: pr, dTBP(4,4)

! Inizio procedimento
pr = deg2rad(psi)

dTBP(1,1) = -l_i*DSIN(pr)
dTBP(1,2) = 0.0d0
dTBP(1,3) = l_i*DCOS(pr)
dTBP(1,4) = 0.0d0
dTBP(2,1) = 0.0d0
dTBP(2,2) = 0.0d0
dTBP(2,3) = 0.0d0
dTBP(2,4) = 0.0d0
dTBP(3,1) = -l_i*DCOS(pr)
dTBP(3,2) = 0.0d0
dTBP(3,3) = -l_i*DSIN(pr)
dTBP(3,4) = 0.0d0
dTBP(4,1) = 0.0d0
dTBP(4,2) = 0.0d0
dTBP(4,3) = 0.0d0
dTBP(4,4) = 0.0d0


END FUNCTION dTBP
! ============================================================

! ============================================================
FUNCTION dRPS(lambda, theta, l_i, lt)

! Calcolo di RPS_A
!
! PARAMETRI DI INPUT
! lambda : angolo dell'ampiezza delle precessione [deg]
! theta  : angolo della rotazione della precessione [deg]
! l_i    :
! lt     :
!
! PARAMETRI DI OUTPUT
! dRPS   : derivata della matrice di rotazione RPS rispetto ad A

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: lambda, theta, l_i
LOGICAL, INTENT(IN) :: lt
REAL(8) :: dRPS(3,3)

! Inizio procedimento
!RPS_A = MATMUL(dRs1(lambda, l_A(omega, t, phi)), Rq(lambda, theta))+&
!        MATMUL(Rs1(lambda), dRq(lambda, theta, l_A(omega, t, phi)))
IF (lt.EQ..TRUE.) THEN
	dRPS = MATMUL(rot(0.d0, theta, 0.d0), dRlx(lambda, l_i))
ELSEIF (lt.EQ..FALSE.) THEN
	dRPS = MATMUL(dRty(theta, l_i), rot(0.d0, 0.d0, lambda))
END IF

END FUNCTION dRPS
! ============================================================

! ============================================================
FUNCTION dRlx(lambda, l_i)

! Calcolo di dRlx
!
! PARAMETRI DI INPUT
! lambda : angolo dell'ampiezza delle precessione [deg]
! l_i    : derivata di lambda rispetto a i, con i = A, omega, phi, dlambda_0, lambda_0
!
! PARAMETRI DI OUTPUT
! dRlx    : derivata della matrice di rotazione R(lambda,x)

IMPLICIT NONE

! Variables
REAL(8) :: lambda, lr, dRlx(3,3)
REAL(8), INTENT(IN) :: l_i

! Inizio procedimento
lr = deg2rad(lambda)

dRlx(1,1) = 0.d0
dRlx(1,2) = 0.d0
dRlx(1,3) = 0.d0
dRlx(2,1) = 0.d0
dRlx(2,2) = -l_i*DSIN(lr)
dRlx(2,3) = -l_i*DCOS(lr)
dRlx(3,1) = 0.d0
dRlx(3,2) = l_i*DCOS(lr)
dRlx(3,3) = -l_i*DSIN(lr)

END FUNCTION dRlx
! ============================================================

! ============================================================
FUNCTION dRty(theta, l_i)

! Calcolo di dRty
!
! PARAMETRI DI INPUT
! theta  : angolo della rotazione della precessione [deg]
! l_i    : derivata di theta rispetto a i, con i = dtheta_0, theta_0
!
! PARAMETRI DI OUTPUT
! dRty    : derivata della matrice di rotazione R(theta,y)

IMPLICIT NONE

! Variables
REAL(8) :: theta, tr, dRty(3,3)
REAL(8), INTENT(IN) :: l_i

! Inizio procedimento
tr = deg2rad(theta)

dRty(1,1) = -l_i*DSIN(tr)
dRty(1,2) = 0.d0
dRty(1,3) = l_i*DCOS(tr)
dRty(2,1) = 0.d0
dRty(2,2) = 0.d0
dRty(2,3) = 0.d0
dRty(3,1) = -l_i*DCOS(tr)
dRty(3,2) = 0.d0
dRty(3,3) = -l_i*DSIN(tr)

END FUNCTION dRty
! ============================================================

! ============================================================
! Derivate seconde
! ============================================================

! ============================================================
FUNCTION d2TPS(lambda, theta, l_i, l_j, l, t)

! Calcolo della derivata della matrice di trasformazione TPS rispetto
! a A, omega, phi, dlambda_0, lambda_0, dtheta_0, theta_0
!
! PARAMETRI DI INPUT
! lambda : angolo dell'ampiezza delle precessione [deg]
! theta  : angolo della rotazione della precessione [deg]
! l_i    :
! l_j    :
! l      :
! t      :
!
! PARAMETRI DI OUTPUT
! d2TPS   : derivata seconda della matrice di trasformazione TPS

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: lambda, theta, l_i, l_j
LOGICAL, INTENT(IN) :: l, t
REAL(8) :: d2TPS(4,4)

! Inizio procedimento
d2TPS(1:3,1:3) = d2RPS(lambda, theta, l_i, l_j, l, t)
d2TPS(1,4) = 0.0d0
d2TPS(2,4) = 0.0d0
d2TPS(3,4) = 0.0d0
d2TPS(4,1) = 0.0d0
d2TPS(4,2) = 0.0d0
d2TPS(4,3) = 0.0d0
d2TPS(4,4) = 0.0d0

END FUNCTION d2TPS
! ============================================================

! ============================================================
FUNCTION d2TBP(l_i, l_j, psi)

! Calcolo della derivata della matrice di trasformazione TBP rispetto a
! dpsi_0 e psi_0
!
! PARAMETRI DI INPUT
! l_i  :
! l_j  :
! psi  : angolo di spin [deg]
!
! PARAMETRI DI OUTPUT
! d2TBP : derivata seconda della matrice di trasformazione TBP

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: l_i, l_j, psi
REAL(8) :: pr, d2TBP(4,4)

! Inizio procedimento
pr = deg2rad(psi)

d2TBP(1,1) = -l_i*l_j*DCOS(pr) !-l_i*DSIN(pr)
d2TBP(1,2) = 0.0d0
d2TBP(1,3) = -l_i*l_j*DSIN(pr) !l_i*DCOS(pr)
d2TBP(1,4) = 0.0d0
d2TBP(2,1) = 0.0d0
d2TBP(2,2) = 0.0d0
d2TBP(2,3) = 0.0d0
d2TBP(2,4) = 0.0d0
d2TBP(3,1) = l_i*l_j*DSIN(pr) !-l_i*DCOS(pr)
d2TBP(3,2) = 0.0d0
d2TBP(3,3) = -l_i*l_j*DCOS(pr) !-l_i*DSIN(pr)
d2TBP(3,4) = 0.0d0
d2TBP(4,1) = 0.0d0
d2TBP(4,2) = 0.0d0
d2TBP(4,3) = 0.0d0
d2TBP(4,4) = 0.0d0


END FUNCTION d2TBP
! ============================================================

! ============================================================
FUNCTION d2RPS(lambda, theta, l_i, l_j, l, t)

! Derivata seconda della matrice di rotazione RPS rispetto ad
! A, omega, phi, dlambda_0, lambda_0, dtheta_0, theta_0
!
! PARAMETRI DI INPUT
! lambda : angolo dell'ampiezza delle precessione [deg]
! theta  : angolo della rotazione della precessione [deg]
! l_i    :
! l_j    :
! l      :
! t      :
!
! PARAMETRI DI OUTPUT
! d2RPS   : calcolo di d2RPS

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: lambda, theta, l_i, l_j
LOGICAL, INTENT(IN) :: l, t
REAL(8) :: d2RPS(3,3)

! Inizio procedimento
!RPS_A = MATMUL(dRs1(lambda, l_A(omega, t, phi)), Rq(lambda, theta))+&
!        MATMUL(Rs1(lambda), dRq(lambda, theta, l_A(omega, t, phi)))
IF ((l.EQ..TRUE.).AND.(t.EQ..FALSE.)) THEN
	d2RPS = MATMUL(rot(0.d0, theta, 0.d0), d2Rlx(lambda, l_i, l_j))
ELSEIF ((l.EQ..FALSE.).AND.(t.EQ..TRUE.)) THEN
	d2RPS = MATMUL(d2Rty(theta, l_i, l_j), rot(0.d0, 0.d0, lambda))
ELSEIF ((l.EQ..TRUE.).AND.(t.EQ..TRUE.)) THEN
	d2RPS = MATMUL(dRty(theta, l_i), dRlx(lambda, l_j))
ELSEIF ((l.EQ..FALSE.).AND.(t.EQ..FALSE.)) THEN
	d2RPS = MATMUL(dRty(theta, l_i), dRlx(lambda, l_j))
END IF

END FUNCTION d2RPS
! ============================================================

! ============================================================
FUNCTION d2Rlx(lambda, l_i, l_j)

! Derivata seconda della matrice di rotazione R(lambda,x)
!
! PARAMETRI DI INPUT
! lambda : angolo dell'ampiezza delle precessione [deg]
! l_i    : derivata di lambda rispetto a i, con i = A, omega, phi, dlambda_0, lambda_0
! l_j    : derivata di lambda rispetto a j, con j = A, omega, phi, dlambda_0, lambda_0
!
! PARAMETRI DI OUTPUT
! d2Rlx  : calcolo di d2Rlx

IMPLICIT NONE

! Variables
REAL(8) :: lambda, lr, d2Rlx(3,3)
REAL(8), INTENT(IN) :: l_i, l_j

! Inizio procedimento
lr = deg2rad(lambda)

d2Rlx(1,1) = 0.d0
d2Rlx(1,2) = 0.d0
d2Rlx(1,3) = 0.d0
d2Rlx(2,1) = 0.d0
d2Rlx(2,2) = -l_i*l_j*DCOS(lr) !-l_i*DSIN(lr)
d2Rlx(2,3) = l_i*l_j*DSIN(lr) !-l_i*DCOS(lr)
d2Rlx(3,1) = 0.d0
d2Rlx(3,2) = -l_i*l_j*DSIN(lr) !l_i*DCOS(lr)
d2Rlx(3,3) = -l_i*l_j*DCOS(lr) !-l_i*DSIN(lr)

END FUNCTION d2Rlx
! ============================================================

! ============================================================
FUNCTION d2Rty(theta, l_i, l_j)

! Derivata seconda della matrice di rotazione R(theta,y)
!
! PARAMETRI DI INPUT
! theta : angolo della rotazione della precessione [deg]
! l_i   : derivata di theta rispetto a i, con i = dtheta_0, theta_0
! l_j   : derivata di theta rispetto a j, con j = dtheta_0, theta_0

! PARAMETRI DI OUTPUT
! d2Rty : calcolo di dRty

IMPLICIT NONE

! Variables
REAL(8) :: theta, tr, d2Rty(3,3)
REAL(8), INTENT(IN) :: l_i, l_j

! Inizio procedimento
tr = deg2rad(theta)

d2Rty(1,1) = -l_i*l_j*DCOS(tr) !-l_i*DSIN(tr)
d2Rty(1,2) = 0.d0
d2Rty(1,3) = -l_i*l_j*DSIN(tr) !l_i*DCOS(tr)
d2Rty(2,1) = 0.d0
d2Rty(2,2) = 0.d0
d2Rty(2,3) = 0.d0
d2Rty(3,1) = l_i*l_j*DSIN(tr) !-l_i*DCOS(tr)
d2Rty(3,2) = 0.d0
d2Rty(3,3) = -l_i*l_j*DCOS(tr) !-l_i*DSIN(tr)

END FUNCTION d2Rty
! ============================================================

END MODULE DerivTrasf_class
!*****
