!****h* TrasfMatrx/Derivates_class
! NAME
! Derivates_class -- Calculates the Jacobian of R
! FUNCTION
! Contains functions used to calculate the Jacobian of the residual vector R
! AUTHOR
! Paolo Zoccarato
! CREATION DATE
! 2006/06/20
! MODIFICATION HISTORY
! +------------------------------------------------------------------+
! |  name      | date   |                  reason                    |
! |------------|--------|--------------------------------------------|
! |            |        |  No one at the moment                      |
! +------------|--------|--------------------------------------------+
! |            |        |                                            |
! +------------------------------------------------------------------+
! COMMANDS
! PUBLIC FUNCTIONS:
! USES
! This module uses:
! the module "Model_class"
! the functions "TSI, TPS, TBP, TBI" of the module "Transform_class"
! the module "DerivTrasf_class"
! NOTES
! Develloped with Compaq Visual Fortran for Windows on Windows XP
! SOURCE

MODULE DerivatesS_class

USE Globals_class, ONLY : dimXc, dimX, dimRed, active, Epoch, Sun, Soy, GPS1, GPS2, Res, mjd2dat,&
                          A, omega, phi, dl0, l0, dth0, th0, dps0, ps0, fb1, fb2, fb3, AssignP
USE ParamS_class, ONLY : Assignament, GetVectS
USE Model_class
USE Transform_class, ONLY : TIS, TPS, TBP, TBS
USE DerivTrasf_class

! Modulo per il calcolo dello jacobiano J(R)
!
! PUBLIC FUNCTIONS:

IMPLICIT NONE

PRIVATE :: Hessian, dw_i_S, d2w_i_S, l_A, l_omega, l_phi

CONTAINS

! ============================================================
SUBROUTINE JR(M, N, X, FJAC, LFJ)

! Calcolo dello Jacobiano del vettore dei parametri R
!
! PARAMETRI DI INPUT
! M    : numero di funzioni (lunghezza di FJAC)
! N    : lunghezza del vettore di stato
! X    : vettore di stato
! LFJ  : dimensione dello Jacobiano
!
! PARAMETRI DI OUTPUT:
! FJAC : Valori dello Jacobiano in X

IMPLICIT NONE

! Variables
INTEGER, INTENT(IN) :: M, N, LFJ
REAL(8), INTENT(IN) :: X(N)
REAL(8), INTENT(OUT) :: FJAC(LFJ, N)
INTEGER :: q
REAL(8) :: alpha, delta, deltaS1, t, t0, g_i1_S(3), g_i2_S(3), dG12_S(3), year0

! Inizio procedimento
CALL mjd2dat(Epoch(1), year0, t0)
DO q = 1, M
	CALL AssignP(X)

	CALL GetVectS(q, t0, alpha, delta, deltaS1, g_i1_S, g_i2_S, t)

	dG12_S = (g_i1_S-g_i2_S)

	FJAC(q, 1:N) =  Jacobian(t, dG12_S(1:3))
END DO

END SUBROUTINE JR
! ============================================================

! ============================================================
SUBROUTINE JJR(M, N, X, FJJAC, LFJ)

! Calcolo delle derivate seconde del vettore dei parametri R
!
! PARAMETRI DI INPUT
! M    : numero di funzioni
! N    : lunghezza del vettore di stato
! X    : vettore di stato
! LFJ  : dimensione dello Jacobiano
!
! PARAMETRI DI OUTPUT:
! FJJAC : Valori delle derivate seconde

IMPLICIT NONE

! Variables
INTEGER, INTENT(IN) :: M, N, LFJ
REAL(8), INTENT(IN) :: X(N)
REAL(8), INTENT(OUT) :: FJJAC(LFJ, N, N)
INTEGER :: q
REAL(8) :: alpha, delta, deltaS1, t, t0, g_i1_S(3), g_i2_S(3), dG12_S(3), year0

! Inizio procedimento
CALL mjd2dat(Epoch(1), year0, t0)
DO q = 1, M
	CALL AssignP(X)

	CALL GetVectS(q, t0, alpha, delta, deltaS1, g_i1_S, g_i2_S, t)

	dG12_S = (g_i1_S-g_i2_S)

	FJJAC(q, 1:N, 1:N) =  Hessian(t, dG12_S(1:3))
END DO

END SUBROUTINE JJR
! ============================================================

! ============================================================
FUNCTION Jacobian(t, dG12S)

!
! PARAMETRI DI INPUT
! t        : tempo [sec]
! Soy      : 
! dG12S    :
!
! PARAMETRI DI OUTPUT:

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: t, dG12S(3)
REAL(8) :: Jacobian(dimX), l_i
INTEGER :: i, count

! Inizio procedimento
count = 1
DO i = 1, dimXc
	IF (i.EQ.1) THEN
		l_i = l_A(omega, t, phi)
	ELSEIF (i.EQ.2) THEN
		l_i = l_omega(A, omega, t, phi)
	ELSEIF (i.EQ.3) THEN
		l_i = l_phi(A, omega, t, phi)
	ELSEIF ((i.EQ.4).OR.(i.EQ.6).OR.(i.EQ.8)) THEN
		l_i = t
	ELSEIF ((i.EQ.5).OR.(i.EQ.7).OR.(i.EQ.9)) THEN
		l_i = 1
	END iF
	IF (active(i).EQV..TRUE.) THEN
		Jacobian(count) = dw_i_S(t, dG12S, l_i, i)
		count = count+1
	END IF
END DO

END FUNCTION Jacobian
! ============================================================

! ============================================================
FUNCTION Hessian(t, dG12S)

!
! PARAMETRI DI INPUT
! t        : tempo [sec]
! dG12S    :
!
! PARAMETRI DI OUTPUT:

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: t, dG12S(3)
REAL(8) :: Hessian(dimX,dimX), l_i, l_j
INTEGER :: i, j, count1, count2

! Inizio procedimento
count1 = 1
DO j = 1, dimXc
	count2 = 1
	IF (j.EQ.1) THEN
		l_j = l_A(omega, t, phi)
	ELSEIF (j.EQ.2) THEN
		l_j = l_omega(A, omega, t, phi)
	ELSEIF (j.EQ.3) THEN
		l_j = l_phi(A, omega, t, phi)
	ELSEIF ((j.EQ.4).OR.(j.EQ.6).OR.(j.EQ.8)) THEN
		l_j = t
	ELSEIF ((j.EQ.5).OR.(j.EQ.7).OR.(j.EQ.9)) THEN
		l_j = 1
	END iF
	IF (active(j).EQV..TRUE.) THEN
		DO i = j, dimXc
			IF (i.EQ.1) THEN
				l_i = l_A(omega, t, phi)
			ELSEIF (i.EQ.2) THEN
				l_i = l_omega(A, omega, t, phi)
			ELSEIF (i.EQ.3) THEN
				l_i = l_phi(A, omega, t, phi)
			ELSEIF ((i.EQ.4).OR.(i.EQ.6).OR.(i.EQ.8)) THEN
				l_i = t
			ELSEIF ((i.EQ.5).OR.(i.EQ.7).OR.(i.EQ.9)) THEN
				l_i = 1
			ELSEIF ((i.GE.10).AND.(i.LE.12)) THEN
				l_i = l_j
			END iF
			IF (active(i).EQV..TRUE.) THEN
				Hessian(count1, count2) = d2w_i_S(t, dG12S, l_i, l_j, i, j)
				Hessian(count2, count1) = d2w_i_S(t, dG12S, l_i, l_j, i, j)
				count2 = count2+1
			END IF
		END DO
		count1 = count1+1
	END IF
END DO

END FUNCTION Hessian
! ============================================================

! ============================================================
REAL(8) FUNCTION dw_i_S(t, dG12S, l_i, d1)

! Calcolo della derivata prima di w_i_S rispetto ad
! A, omega, phi, dlambda_0, lambda_0, dtheta_0, theta_0,
! dpsi_0, psi_0, f_b_1, f_b_2 e f_b_3
!
! PARAMETRI DI INPUT
! t     : tempo [sec]
! dG12S :
! l_i   :
! lt    :
!
! PARAMETRI DI OUTPUT:
! dw_i_S : deriavta di w_i

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: t, dG12S(3), l_i
INTEGER, INTENT(IN) :: d1
REAL(8) :: df_S_tmp(4), df_S(3), f_B(3), lambda, theta, psi

! Inizio procedimento
CALL Assignament(t, f_B, lambda, theta, psi)

IF (d1.LE.5) THEN
	df_S_tmp = MATMUL(dTPS(lambda, theta, l_i, .TRUE.), MATMUL(TBP(psi), (/ f_B, 1.d0 /)))
ELSEIF ((d1.EQ.6).OR.(d1.EQ.7)) THEN
	df_S_tmp = MATMUL(dTPS(lambda, theta, l_i, .FALSE.), MATMUL(TBP(psi), (/ f_B, 1.d0 /)))
ELSEIF ((d1.EQ.8).OR.(d1.EQ.9)) THEN
	df_S_tmp = MATMUL(TPS(lambda, theta), MATMUL(dTBP(psi, l_i), (/ f_B, 1.d0 /)))
ELSEIF (d1.LE.10) THEN
	CALL TBS(lambda, theta, psi, (/ 1.d0, 0.d0, 0.d0 /), df_S)
ELSEIF (d1.LE.11) THEN
	CALL TBS(lambda, theta, psi, (/ 0.d0, 1.d0, 0.d0 /), df_S)
ELSEIF (d1.LE.12) THEN
	CALL TBS(lambda, theta, psi, (/ 0.d0, 0.d0, 1.d0 /), df_S)
ELSE
	WRITE(6,*) 'First derivate range error!'
	PRINT *, 'First derivate range error!'
END IF
df_S = df_S_tmp(1:3)
dw_i_S = DOT_PRODUCT(df_S, dG12S)

END FUNCTION dw_i_S
! ============================================================

! ============================================================
REAL(8) FUNCTION d2w_i_S(t, dG12S, l_i, l_j, d1, d2)

! Calcolo della derivata seconda di w_i_S rispetto ad
! A, omega, phi, dlambda_0, lambda_0, dtheta_0, theta_0,
! dpsi_0, psi_0, f_b_1, f_b_2 e f_b_3
!
! PARAMETRI DI INPUT
! t       : tempo [sec]
! dG12S   :
! l_i     :
! l_j     :
! d1      :
! d2      :
!
! PARAMETRI DI OUTPUT:
! d2w_i_S : deriavta seconda di w_i

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: t, dG12S(3), l_i, l_j
INTEGER, INTENT(IN) :: d1, d2
REAL(8) :: df_S_tmp(4), df_S(3), f_B(3), lambda, theta, psi

! Inizio procedimento
CALL Assignament(t, f_B, lambda, theta, psi)

IF ((d1.LE.5).AND.(d2.LE.5)) THEN
	df_S_tmp = MATMUL(d2TPS(lambda, theta, l_i, l_j, .TRUE., .FALSE.), MATMUL(TBP(psi), (/ f_B, 1.d0 /)))
ELSEIF (((d1.LE.5).AND.((d2.GE.6).AND.(d2.LE.7))).OR.(((d1.GE.6).AND.(d1.LE.7)).AND.(d2.LE.5))) THEN
	df_S_tmp = MATMUL(d2TPS(lambda, theta, l_i, l_j, .TRUE., .TRUE.), MATMUL(TBP(psi), (/ f_B, 1.d0 /)))
ELSEIF (((d1.LE.5).AND.((d2.GE.8).AND.(d2.LE.9))).OR.((d2.LE.5).AND.((d1.GE.8).AND.(d1.LE.9)))) THEN
	df_S_tmp = MATMUL(dTPS(lambda, theta, l_i, .TRUE.), MATMUL(dTBP(psi, l_j), (/ f_B, 1.d0 /)))
ELSEIF (((d1.LE.5).AND.(d2.EQ.10)).OR.((d2.LE.5).AND.(d1.EQ.10))) THEN
	df_S_tmp = MATMUL(dTPS(lambda, theta, l_i, .TRUE.), MATMUL(TBP(psi), (/ 1.d0, 0.d0, 0.d0, 1.d0 /)))
ELSEIF (((d1.LE.5).AND.(d2.EQ.11)).OR.((d2.LE.5).AND.(d1.EQ.11))) THEN
	df_S_tmp = MATMUL(dTPS(lambda, theta, l_i, .TRUE.), MATMUL(TBP(psi), (/ 0.d0, 1.d0, 0.d0, 1.d0 /)))
ELSEIF (((d1.LE.5).AND.(d2.EQ.12)).OR.((d2.LE.5).AND.(d1.EQ.12))) THEN
	df_S_tmp = MATMUL(dTPS(lambda, theta, l_i, .TRUE.), MATMUL(TBP(psi), (/ 0.d0, 0.d0, 1.d0, 1.d0 /)))
ELSEIF (((d1.GE.6).AND.(d1.LE.7)).AND.((d2.GE.6).AND.(d2.LE.7))) THEN
	df_S_tmp = MATMUL(d2TPS(lambda, theta, l_i, l_j, .FALSE., .TRUE.), MATMUL(TBP(psi), (/ f_B, 1.d0 /)))
ELSEIF ((((d1.GE.6).AND.(d1.LE.7)).AND.((d2.GE.8).AND.(d2.LE.9))).OR.(((d2.GE.6).AND.(d2.LE.7)).AND.((d1.GE.8).AND.(d1.LE.9)))) THEN
	df_S_tmp = MATMUL(dTPS(lambda, theta, l_i, .FALSE.), MATMUL(dTBP(psi, l_j), (/ f_B, 1.d0 /)))
ELSEIF ((((d1.GE.6).AND.(d1.LE.7)).AND.(d2.EQ.10)).OR.(((d2.GE.6).AND.(d2.LE.7)).AND.(d1.EQ.10))) THEN
	df_S_tmp = MATMUL(dTPS(lambda, theta, l_i, .FALSE.), MATMUL(TBP(psi), (/ 1.d0, 0.d0, 0.d0, 1.d0 /)))
ELSEIF ((((d1.GE.6).AND.(d1.LE.7)).AND.(d2.EQ.11)).OR.(((d2.GE.6).AND.(d2.LE.7)).AND.(d1.EQ.11))) THEN
	df_S_tmp = MATMUL(dTPS(lambda, theta, l_i, .FALSE.), MATMUL(TBP(psi), (/ 0.d0, 1.d0, 0.d0, 1.d0 /)))
ELSEIF ((((d1.GE.6).AND.(d1.LE.7)).AND.(d2.EQ.12)).OR.(((d2.GE.6).AND.(d2.LE.7)).AND.(d1.EQ.12))) THEN
	df_S_tmp = MATMUL(dTPS(lambda, theta, l_i, .FALSE.), MATMUL(TBP(psi), (/ 0.d0, 0.d0, 1.d0, 1.d0 /)))
ELSEIF (((d1.GE.8).AND.(d1.LE.9)).AND.((d2.GE.8).AND.(d2.LE.9))) THEN
	df_S_tmp = MATMUL(TPS(lambda, theta), MATMUL(d2TBP(psi, l_i, l_j), (/ f_B, 1.d0 /)))
ELSEIF ((((d1.GE.8).AND.(d1.LE.9)).AND.(d2.EQ.10)).OR.(((d2.GE.8).AND.(d2.LE.9)).AND.(d1.EQ.10))) THEN
	df_S_tmp = MATMUL(TPS(lambda, theta), MATMUL(dTBP(psi, l_i), (/ 1.d0, 0.d0, 0.d0, 1.d0 /)))
ELSEIF ((((d1.GE.8).AND.(d1.LE.9)).AND.(d2.EQ.11)).OR.(((d2.GE.8).AND.(d2.LE.9)).AND.(d1.EQ.11))) THEN
	df_S_tmp = MATMUL(TPS(lambda, theta), MATMUL(dTBP(psi, l_i), (/ 0.d0, 1.d0, 0.d0, 1.d0 /)))
ELSEIF ((((d1.GE.8).AND.(d1.LE.9)).AND.(d2.EQ.12)).OR.(((d2.GE.8).AND.(d2.LE.9)).AND.(d1.EQ.12))) THEN
	df_S_tmp = MATMUL(TPS(lambda, theta), MATMUL(dTBP(psi, l_i), (/ 0.d0, 0.d0, 1.d0, 1.d0 /)))
ELSEIF (((d1.GE.10).AND.(d1.LE.12)).AND.((d2.GE.10).AND.(d2.LE.12))) THEN
	CALL TBS(lambda, theta, psi, (/ 0.d0, 0.d0, 0.d0 /), df_S)
ELSE
	WRITE(6,*) 'Second derivate range error!'
	WRITE(6,*) d1, d2
	PRINT *, 'Second derivate range error!'
	PRINT *, d1, d2
END IF
df_S = df_S_tmp(1:3)
d2w_i_S = DOT_PRODUCT(df_S(1:3), dG12S)

END FUNCTION d2w_i_S
! ============================================================

! ============================================================
REAL(8) FUNCTION l_A(omega, t, phi)

! Calcolo di l_A
!
! PARAMETRI DI INPUT
! omega  : frequenza della variazione dell'ampiezza del moto di precessione [deg/s]
! t      : tempo [sec]
! phi    : fase della variazione dell'ampiezza del moto di precessione [deg]
!
! PARAMETRI DI OUTPUT
! l_A    : derivata di lambda rispetto ad A

IMPLICIT NONE

! Variables
REAL(8) :: phi, omega, pr, or
REAL(8), INTENT(IN) :: t

! Inizio procedimento
pr = deg2rad(phi)
or = deg2rad(omega)

l_A = DSIN(or*t+pr)

END FUNCTION l_A
! ============================================================

! ============================================================
REAL(8) FUNCTION l_omega(A, omega, t, phi)

! Calcolo di l_omega
!
! PARAMETRI DI INPUT
! A       : ampiezza del moto di precessione [deg]
! omega   : frequenza della variazione dell'ampiezza del moto di precessione [deg/s]
! t       : tempo [sec]
! phi     : fase della variazione dell'ampiezza del moto di precessione [deg]
!
! PARAMETRI DI OUTPUT
! l_omega : derivata di lambda rispetto a omega

IMPLICIT NONE

! Variables
REAL(8) :: phi, omega, pr, or
REAL(8), INTENT(IN) :: A, t

! Inizio procedimento
pr = deg2rad(phi)
or = deg2rad(omega)

l_omega = A*DCOS(or*t+pr)*t

END FUNCTION l_omega
! ============================================================

! ============================================================
REAL(8) FUNCTION l_phi(A, omega, t, phi)

! Calcolo di l_phi
!
! PARAMETRI DI INPUT
! A     : ampiezza del moto di precessione [deg]
! omega : frequenza della variazione dell'ampiezza del moto di precessione [deg/s]
! t     : tempo [sec]
! phi   : fase della variazione dell'ampiezza del moto di precessione [deg]
!
! PARAMETRI DI OUTPUT
! l_phi : derivata di lambda rispetto a phi

IMPLICIT NONE

! Variables
REAL(8) :: phi, omega, pr, or
REAL(8), INTENT(IN) :: A, t

! Inizio procedimento
pr = deg2rad(phi)
or = deg2rad(omega)

l_phi = A*DCOS(or*t+pr)

END FUNCTION l_phi
! ============================================================

END MODULE DerivatesS_class
!*****
