MODULE LeastSquares_class

USE Numerical_libraries
USE Globals_class, ONLY : brute, dimX, dimRed, rms, AssignP, fb1, fb2, fb3
USE Param_class
USE Derivates_class

! Modulo per l'applicazione del medodo dei Minimi Quadrati

IMPLICIT NONE

PRIVATE :: LeaSquaNonLin

! ============================================================
INTERFACE LeastSquares

MODULE PROCEDURE LeaSquaNonLin

END INTERFACE LeastSquares
! ============================================================

CONTAINS

! ============================================================
SUBROUTINE LeaSquaNonLin(X, TOLLs, TOLL, m)

! realizzazione del metodo di soluzione del problema dei Minimi Quadrati non lineare con SVD

IMPLICIT NONE

! Variables
INTEGER, INTENT(IN) :: m
REAL(8), INTENT(IN) :: TOLLs, TOLL
REAL(8), INTENT(INOUT) :: X(dimX)
INTEGER :: p, q, HSr !, i, j, Nr
REAL(8) :: RMS, Sig0 !, V(dimX), Cov(dimX,dimX), devSt(dimX), rho(dimX,dimX)
REAL(8) :: Rx(dimRed), JRx(dimRed,dimX), JJRx(dimRed,dimX,dimX), HR(dimX,dimX)
REAL(8) :: SR(2), GS(dimX), HS(dimX,dimX), InvHS(dimX,dimX), s(dimX) !, N(dimX,dimX), InvN(dimX,dimX)
COMPLEX(8) :: EVAL(dimX-erase)

! Inizio procedimento
DO p = 1, m
	CALL AssignP(X)

	CALL R(dimRed, dimX, X, Rx)
	SR(2) = 0.5d0*DOT_PRODUCT(Rx,Rx)
	IF (p>1) THEN
		IF (SR(1)<SR(2)) THEN
			WRITE(6,*) 'Exit mode 1'
			X = X-s
			EXIT
		END IF
	END IF
	CALL JR(dimRed, dimX, X, JRx, dimRed)
	CALL JJR(dimRed, dimX, X, JJRx, dimRed)

	! S(R)
	SR(1) = SR(2)

	! Gradiente di S(R)
    GS = MATMUL(TRANSPOSE(JRx), Rx)

	DO q = 1, dimX
		HR(:,q) = MATMUL(TRANSPOSE(JJRx(:,:,q)), Rx)
	END DO

    ! Hessiano di S(R)
	HS = MATMUL(TRANSPOSE(JRx), JRx)+HR
	
    ! Calcolo dei valori singolari di HS
	CALL DEVLRG (dimX, HS, dimX, EVAL)

	! Calcolo della pseudoinversa di HS
	CALL DLSGRR (dimX, dimX, HS, dimX, 1.d-20, HSr, InvHS, dimX)
	s = MATMUL(InvHS, -GS)

    ! Root Mean Square
	RMS = SQRT(DOT_PRODUCT(Rx, Rx)/(dimRed-dimX))

    ! Varianza unitaria
	Sig0 = DOT_PRODUCT(Rx, Rx)/(dimRed-dimX)

!   ! Caclolo matrice di covarianza
!	N = MATMUL(TRANSPOSE(JRx),JRx)

!	CALL DLSGRR(dimX, dimX, N, dimX, 1.d-20, Nr, InvN, dimX)
!	CALL DLINRG(dimX, N, dimX, InvN, dimX)

!	Cov = RMS*InvN
	
!	DO i = 1, dimX
!		devSt(i) = SQRT(Cov(i,i))
!	END DO
!	DO i = 1, dimX
!		DO j = 1, dimX
!			rho(i,j) = Cov(i,j)/(devSt(i)*devSt(j))
!		END DO
!	END DO

    IF (brute.EQ..FALSE.) THEN
		WRITE(6,*) 'p : ', p
		CALL DWRCRN ('EVAL', dimX, 1, EVAL, dimX, 0)
		WRITE(6,*)
		WRITE(6,*) 'HS rank : ', HSr
		WRITE(6,*)
		WRITE(6,*) 'RMS : ', RMS
		WRITE(6,*)
		WRITE(6,*) 'Var0 : ', Sig0
		WRITE(6,*)
		WRITE(6,*) 'SR : ', SR(1)
!		WRITE(6,*)
!		CALL DWRRRN ('Covariance', dimX, dimX, Cov, dimX, 0)
!		CALL DWRRRN ('Dev Stand', dimX, 1, DevSt, dimX, 0)
!		CALL DWRRRN ('Indici correlaz', dimX, dimX, rho, dimX, 0)
		CALL DWRRRN ('s', dimX, 1, s, dimX, 0)
		WRITE(6,*)
		WRITE(6,*) 'Norm s : ', MAXVAL(s/X) !SQRT(DOT_PRODUCT(s, s))
!		CALL DWRRRN ('X', dimX, 1, X, dimX, 0)
		WRITE(6,*)
		WRITE(6, FMT='(F15.10)') X
		WRITE(6,*)

!		IF (SQRT(DOT_PRODUCT(s, s))<TOLL) EXIT	
		IF (RMS<TOLL) THEN
			WRITE(6,*) 'Exit mode 2'
			EXIT
		END IF

		IF (MAXVAL(s/X)<TOLLs) THEN
			WRITE(6,*) 'Exit mode 3'
			EXIT
		END IF

		X = X+s
	ELSE
		WRITE(6,100) X, HSr, RMS, Sig0, SR(1)
		100 FORMAT(8F10.5, I2, 2F45.40, 2X, F45.40)
	END IF

END DO

!WRITE(8, FMT='(4F35.30)') fb1, fb2, fb3, RMS

END SUBROUTINE LeaSquaNonLin
! ============================================================

END MODULE LeastSquares_class
