MODULE LeastSquares_class

USE Numerical_libraries
USE Globals_class, ONLY : dimX, dimRed, rms, AssignP
USE Vectors_class
USE AsceDecl_class
USE Param_class
USE Derivates_class

! Modulo per l'applicazione del medodo dei Minimi Quadrati

IMPLICIT NONE

CONTAINS

! ============================================================
SUBROUTINE LeaSquaLin(X, TOLL, m)

! realizzazione del metodo dei Minimi Quadrati tramite SVD

! Variables
INTEGER, INTENT(IN) :: m
REAL(8), INTENT(IN) :: TOLL
REAL(8), INTENT(INOUT) :: X(dimX)
INTEGER :: p, q, i, j, HSr
REAL(8) :: VarSquare, ChiSquare, Cov(dimX,dimX), devSt(dimX), rho(dimX,dimX)
REAL(8) :: Rx(dimRed), JRx(dimRed,dimX), JJRx(dimRed,dimX,dimX), HR(dimX,dimX)
REAL(8) :: SR, GS(dimX), HS(dimX,dimX), s(dimX) !, U(dimRed,dimX), V(dimX,dimX), Sig(dimX)
COMPLEX(8) :: EVAL(dimX-erase)

! Inizio procedimento
DO p = 1, m
	WRITE(6,*) 'p : ', p
	CALL AssignP(X)
	CALL R(dimRed, dimX, X, Rx)
	CALL JR(dimRed, dimX, X, JRx, dimRed)
	CALL JJR(dimRed, dimX, X, JJRx, dimRed)

	! S(R)
	SR = DOT_PRODUCT(Rx, Rx)

	! Varianza
	VarSquare = SR/(dimRed-dimX)


	! Gradiente di S(R)
    GS = 2.d0*MATMUL(TRANSPOSE(JRx), Rx)

	DO q = 1, dimX
		HR(1:dimX,q) = 2*MATMUL(TRANSPOSE(JJRx(1:dimRed,1:dimX,q)), Rx)
	END DO

    ! Hessiano di S(R)
	HS = 2.d0*MATMUL(TRANSPOSE(JRx), JRx) !+HR

	CALL DEVLRG (dimX, HS, dimX, EVAL)
	CALL DWRCRN ('EVAL', dimX, 1, EVAL, dimX, 0)

!	CALL DLINRG (dimX, HS, dimX, Cov, dimX)

	CALL DLSGRR (dimX, dimX, HS, dimX, 1.d-20, HSr, Cov, dimX)
	s = MATMUL(Cov, -GS)

	X = X+s
	
	DO i = 1, dimX
		devSt(i) = SQRT(Cov(i,i))
	END DO
	DO i = 1, dimX
		DO j = 1, dimX
			rho(i,j) = Cov(i,j)/(devSt(i)*devSt(j))
		END DO
	END DO

	ChiSquare = DOT_PRODUCT(Rx/VarSquare, Rx/VarSquare)
	
	CALL DWRRRN ('s', dimX, 1, s, dimX, 0)
	CALL DWRRRN ('Covariance', dimX, dimX, Cov, dimX, 0)
	CALL DWRRRN ('Dev Stand', dimX, 1, DevSt, dimX, 0)
	WRITE(6,*)
	WRITE(6,*) 'HS rank : ', HSr
	WRITE(6,*)
	WRITE(6,*) 'VarSquare : ', VarSquare
	WRITE(6,*)
	WRITE(6,*) 'ChiSquare : ', ChiSquare
	WRITE(6,*)
	CALL DWRRRN ('X', dimX, 1, X, dimX, 0)
	CALL DWRRRN ('Indici correlaz', dimX, dimX, rho, dimX, 0)

	IF (ChiSquare<TOLL) EXIT
END DO

END SUBROUTINE LeaSquaLin
! ============================================================

END MODULE LeastSquares_class