! ============================================================
SUBROUTINE LevenMarq(XLM, XF, IPARAM)

USE Numerical_libraries
USE Globals_class, ONLY : dimX, dimRed, AssignP

! soluzione del problema ai minimi quadrati non lineare con il metodo di Levenberg-Marquardt

IMPLICIT NONE

! Variabili
REAL(8), INTENT(IN) :: XLM(dimX)
INTEGER, INTENT(OUT) :: IPARAM(6)
REAL(8), INTENT(OUT) :: XF(dimX)
REAL(8) :: RPARAM(7), resid(dimRed), FJAC(dimRed, dimX), XSCALE(dimX), FSCALE(dimRed)
EXTERNAL :: R, JR

! Inizio procedimento
CALL AssignP(XLM)

CALL DU4LSF(IPARAM, RPARAM)
IPARAM(4) = 600
IPARAM(5) = 600
IPARAM(6) = 1
XSCALE = 1.d0
FSCALE = 1.d0

CALL DUNLSJ(R, JR, dimRed, dimX, XLM, XSCALE, FSCALE, IPARAM, RPARAM, XF, resid, FJAC, dimRed)

END SUBROUTINE LevenMarq
! ============================================================

! ============================================================
SUBROUTINE LMB(XLM, XF, IPARAM)

USE Numerical_libraries
USE Globals_class, ONLY : dimX, dimRed, AssignP, XLB, XUB

! soluzione del problema ai minimi quadrati non lineare con il metodo di Levenberg-Marquardt

IMPLICIT NONE

! Variabili
REAL(8), INTENT(IN) :: XLM(dimX)
INTEGER, INTENT(OUT) :: IPARAM(6)
REAL(8), INTENT(OUT) :: XF(dimX)
REAL(8) :: RPARAM(7), XSCALE(dimX), FSCALE(dimRed), resid(dimRed), FJAC(dimRed)
EXTERNAL :: R

! Inizio procedimento
CALL AssignP(XLM)

!CALL DU4LSF(IPARAM, RPARAM)
!IPARAM(4) = 600
!IPARAM(5) = 600
!IPARAM(6) = 1
IPARAM(1) = 0
XSCALE = 1.d0
FSCALE = 1.d0

CALL DBCLSF(R, dimRed, dimX, XLM, 0, XLB, XUB, XSCALE, FSCALE, IPARAM, RPARAM, XF, resid, FJAC, dimRed)

END SUBROUTINE LMB
! ============================================================

! ============================================================
SUBROUTINE LevenMarqB(X, XFB, IPARAM)

USE Numerical_libraries
USE Globals_class, ONLY : dimX, dimRed, AssignP, XLB, XUB

! soluzione del problema ai minimi quadrati non lineare con il metodo di Levenberg-Marquardt

IMPLICIT NONE

! Variabili
REAL(8), INTENT(IN) :: X(dimX)
INTEGER, INTENT(OUT) :: IPARAM(6)
REAL(8), INTENT(OUT) :: XFB(dimX)
REAL(8) :: RPARAM(7), resid(dimRed), FJAC(dimRed, dimX), XSCALE(dimX), FSCALE(dimRed)
EXTERNAL :: R, JR

! Inizio procedimento
CALL AssignP(X)

CALL DU4LSF(IPARAM, RPARAM)
IPARAM(4) = 600
IPARAM(5) = 600
IPARAM(6) = 1
XSCALE = 1.d0
FSCALE = 1.d0

CALL DBCLSJ(R, JR, dimRed, dimX, X, 0, XLB, XUB, XSCALE, FSCALE, IPARAM, RPARAM, XFB, resid, FJAC, dimRed)

END SUBROUTINE LevenMarqB
! ============================================================

! ============================================================
SUBROUTINE R(M, N, X, F)

USE Globals_class, ONLY : Epoch, Sun, Soy, GPS1, GPS2, Res, mjd2dat, AssignP
USE Param_class, ONLY : w, GetVectI

! Calcolo del vettore dei parametri R
!
! PARAMETRI DI INPUT
! M : numero di funzioni (lunghezza di F)
! N : lunghezza del vettore di stato
! X : vettore di stato
!
! PARAMETRI DI OUTPUT:
! F : Valori delle funzioni in X

IMPLICIT NONE

! Variabili
INTEGER, INTENT(IN) :: M, N
REAL(8), INTENT(IN) :: X(N)
REAL(8), INTENT(OUT) :: F(M)
INTEGER :: q
REAL(8) :: alpha, delta, deltaS1, t, t0, g_i1_I(3), g_i2_I(3), year0

! Inizio procedimento
CALL mjd2dat(Epoch(1), year0, t0)
DO q = 1, M
	CALL AssignP(X)
	
	CALL GetVectI(q, t0, alpha, delta, deltaS1, g_i1_I, g_i2_I, t)
	
	F(q) =  w(alpha, delta, deltaS1, t, g_i1_I, g_i2_I, Res(q, 3))
END DO

END SUBROUTINE R
! ============================================================

! ============================================================
SUBROUTINE JR(M, N, X, FJAC, LFJ)

USE Globals_class, ONLY : Epoch, Sun, Soy, GPS1, GPS2, Res, mjd2dat, AssignP
USE Param_class, ONLY : GetVectI
USE Derivates_class, ONLY : Jacobian

! Calcolo dello Jacobiano del vettore dei parametri R
!
! PARAMETRI DI INPUT
! M    : numero di funzioni (lunghezza di FJAC)
! N    : lunghezza del vettore di stato
! X    : vettore di stato
! LFJ  : dimensione dello Jacobiano
!
! PARAMETRI DI OUTPUT:
! FJAC : Valori dello Jacobiano in X

IMPLICIT NONE

! Variables
INTEGER, INTENT(IN) :: M, N, LFJ
REAL(8), INTENT(IN) :: X(N)
REAL(8), INTENT(OUT) :: FJAC(LFJ, N)
INTEGER :: q
REAL(8) :: alpha, delta, deltaS1, t, t0, g_i1_I(3), g_i2_I(3), dG12S(3), year0

! Inizio procedimento
CALL mjd2dat(Epoch(1), year0, t0)
DO q = 1, M
	CALL AssignP(X)

	CALL GetVectI(q, t0, alpha, delta, deltaS1, g_i1_I, g_i2_I, t)

	dG12S = (g_i1_I-g_i2_I)

	FJAC(q, 1:N) = Jacobian(alpha, delta, deltaS1, t, dG12S)
END DO

END SUBROUTINE JR
! ============================================================