MODULE Model_class

USE Globals_class, ONLY : deg2rad

! Modulo che implementa il modello dell'assetto della Soyuz
!
! PUBLIC FUNCTIONS:
! lambda_calc(A, omega, t, phi, dlambda_0, lambda_0)
! theta_calc(t, dtheta_0, theta_0)
! psi_calc(t, dpsi_0, psi_0)

IMPLICIT NONE

CONTAINS

! ============================================================
REAL(8) FUNCTION lambda_calc(A, omega, t, phi, dlambda_0, lambda_0)

! Calcola lambda secondo il modello
!
! PARAMETRI DI INPUT:
! A         : ampiezza del moto di precessione [deg]
! omega     : frequenza della variazione dell'ampiezza del moto di precessione [deg/s]
! t         : tempo [sec]
! phi       : fase della variazione dell'ampiezza del moto di precessione [deg]
! dlambda_0 : [deg/s]
! lambda_0  : [deg]
!
!PARAMETRI DI PUTPUT:
! lambda    : angolo dell'ampiezza della precessione [deg]

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: A, t, dlambda_0, lambda_0
REAL(8) :: s, omega, phi

! Inizio procedimento
omega = deg2rad(omega)
phi = deg2rad(phi)

s = DSIN(omega*t+phi)
lambda_calc = A*s+dlambda_0*t+lambda_0

END FUNCTION lambda_calc
! ============================================================

! ============================================================
REAL(8) FUNCTION theta_calc(t, dtheta_0, theta_0)

! Calcola theta secondo il modello
!
! PARAMETRI DI INPUT:
! t        : tempo [sec]
! dtheta_0 : [deg/s]
! theta_0  : [deg]
!
!PARAMETRI DI PUTPUT:
! theta    : angolo di rotazione della precessione [deg]

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: t, dtheta_0, theta_0

! Inizio procedimento
theta_calc = dtheta_0*t+theta_0

END FUNCTION theta_calc
! ============================================================

! ============================================================
REAL(8) FUNCTION psi_calc(t, dpsi_0, psi_0)

! Calcola theta secondo il modello
!
! PARAMETRI DI INPUT:
! t      : tempo [sec]
! dpsi_0 : [deg/s]
! psi_0  : [deg]
!
!PARAMETRI DI PUTPUT:
! psi    : angolo di rotazione della precessione [deg]

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: t, dpsi_0, psi_0

! Inizio procedimento
psi_calc = dpsi_0*t+psi_0

END FUNCTION psi_calc
! ============================================================

END MODULE Model_class
