MODULE Param_class

USE Globals_class, ONLY : brute, lambda1, dimX, erase, Epoch, Sun, Soy, GPS1, GPS2, Res, mjd2dat,&
						  A, omega, phi, dl0, l0, dth0, th0, dps0, ps0, fb1, fb2, fb3, AssignP, tunnel
USE AsceDecl_class
USE Vectors_class
USE Model_class
USE Transform_class, ONLY : TBI

! Modulo per il calcolo del vettore degli scarti R
!
! PUBLIC FUNCTIONS:
! R(M, N, X, F)

IMPLICIT NONE

PRIVATE :: uv

CONTAINS

! ============================================================
SUBROUTINE R(M, N, X, F)

! Calcolo del vettore dei parametri R
!
! PARAMETRI DI INPUT
! M : numero di funzioni (lunghezza di F)
! N : lunghezza del vettore di stato
! X : vettore di stato
!
! PARAMETRI DI OUTPUT:
! F : Valori delle funzioni in X

IMPLICIT NONE

! Variables
INTEGER, INTENT(IN) :: M, N
REAL(8), INTENT(IN) :: X(N)
REAL(8), INTENT(OUT) :: F(M)
INTEGER :: q
REAL(8) :: alpha, delta, deltaS1, t, t0, g_i1_I(3), g_i2_I(3), year0

! Inizio procedimento
CALL mjd2dat(Epoch(1), year0, t0)
DO q = 1, M
	CALL AssignP(X)

	CALL GetVectI(q, t0, alpha, delta, deltaS1, g_i1_I, g_i2_I, t)
	
	F(q) =  w(alpha, delta, deltaS1, t, g_i1_I, g_i2_I, Res(q, 3))
END DO

END SUBROUTINE R
! ============================================================

! ============================================================
REAL(8) FUNCTION w(alpha, delta, deltaS1, t, g_i1_I, g_i2_I, resid)

! Calcolo di w
!
! PARAMETRI DI INPUT
! alpha  : ascensione retta del versore s2 rispetto al sdr I [deg]
! delta  : declinazione del versore s2 rispetto al sdr I [deg]
! t      : tempo [sec]
! g_i1_I :
! g_i2_I :
! resid  :
!
! PARAMETRI DI OUTPUT:
! w      : 

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: alpha, delta, deltaS1, t, g_i1_I(3), g_i2_I(3), resid
real(8) :: u, v, model, m

! Inizio procedimento
u = uv(alpha, delta, deltaS1, t, g_i1_I)
v = uv(alpha, delta, deltaS1, t, g_i2_I)
model = (u-v)
m = lambda1*resid
w = model-m

!WRITE (8,10) u
!10 FORMAT("u : ", F20.10)
!WRITE (8,11) v
!11 FORMAT("v : ", F20.10)
!WRITE (8,12) model, m, w
!12 FORMAT("Modello : ", F20.10, ", Residuo : ", F20.10, " Deviazione : ", F20.10)
IF ((tunnel.EQ..FALSE.).AND.(brute.EQ..FALSE.)) THEN
	WRITE (7,*) model, m, w
END IF

END FUNCTION w
! ============================================================

! ============================================================
REAL(8) FUNCTION uv(alpha, delta, deltaS1, t, g_i_I)

! Calcolo di u o di v, proiezioni del vettore congiungente il CM e il CF della Soyuz f_I rispettivamente sul
! versore che dal CM della Spyuz punta al primo satellite GPS g_i1_I e sul versore che dal CM della Soyuz
! punta al secondo satellite GPS g_i2_I, sempre restando nel sdr inerziale equatoriale
!
! PARAMETRI DI INPUT
! alpha  : ascensione retta del versore s2 rispetto al sdr I [deg]
! delta  : declinazione del versore s2 rispetto al sdr I [deg]
! t      : tempo [sec]
! X      : vettore di stato contenette: A, omega, phi, dtheta_0, theta_0,
!          dlambda_0, lambda_0, dpsi_0, psi_0, f_b_1, f_b_2, f_b_3 
! g_i_I  : versore che dal CM della Soyuz punta a uno dei satelliti GPS coinvolti nel calcolo delle doppie differenze
!
! PARAMETRI DI OUTPUT:
! uv      : proiezione di f_I su g_i_I

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: alpha, delta, deltaS1, t, g_i_I(3)
REAL(8) :: f_I(3), f_B(3), lambda, theta, psi

! Inizio procedimento
CALL Assignament(t, f_B, lambda, theta, psi)

CALL TBI(alpha, delta, deltaS1, lambda, theta, psi, (/ 0.d0, 0.d0, 0.d0 /), f_B, f_I)

uv = DOT_PRODUCT(f_I, g_i_I)

END FUNCTION uv
! ============================================================

! ============================================================
SUBROUTINE Assignament(t, f_B, lambda, theta, psi)

! Assegnai valori contenuti nel vettore di stato X alle specifiche variabili
!
! PARAMETRI DI INPUT:
! t         : tempo [sec]
!
! PARAMETRI DI OUTPUT:
! f_B       :
! lambda    :
! theta     :
! psi       :

! Variables
REAL(8), INTENT(IN) :: t
REAL(8), INTENT(OUT) :: f_B(3), lambda, theta, psi

! Inizio procedimento
f_B = (/ fb1, fb2, fb3 /)
lambda = lambda_calc(A, omega, t, phi, dl0, l0)
theta = theta_calc(t, dth0, th0)
psi = psi_calc(t, dps0, ps0)

END SUBROUTINE Assignament
! ============================================================

! ============================================================
SUBROUTINE GetVectI(q, t0, alpha, delta, deltaS1, g_i1_I, g_i2_I, t)

! 

IMPLICIT NONE 

! Variables
INTEGER, INTENT(IN) :: q
REAL(8), INTENT(IN) :: t0
REAL(8), INTENT(OUT) :: alpha, delta, deltaS1, g_i1_I(3), g_i2_I(3), t
REAL(8) :: s_2(3), year, epsilon

! Inizio procedimento
CALL mjd2dat(Epoch(q), year, t)

s_2 = s2(Sun(q, 1:3), Soy(q, 1:3))

alpha = AR(s_2)
delta = Dec(s_2)
IF (t.EQ.t0) THEN
	IF (delta.EQ.0) THEN
		WRITE(*,*) 'Error: zero Sun declination! Select different inital date.'
		STOP
	END IF
	epsilon = Eps(alpha, delta)
END IF
deltaS1 = Decli(alpha, epsilon)

g_i1_I = giI(GPS1(q, 2:4), Soy(q, 1:3))
g_i2_I = giI(GPS2(q, 2:4), Soy(q, 1:3))

t = t-t0

END SUBROUTINE GetVectI
! ============================================================

END MODULE Param_class