MODULE ParamS_class

USE Globals_class, ONLY : brute, lambda1, dimX, erase, Epoch, Sun, Soy, GPS1, GPS2, Res, mjd2dat,&
						  A, omega, phi, dl0, l0, dth0, th0, dps0, ps0, fb1, fb2, fb3, AssignP, tunnel
USE AsceDecl_class
USE Vectors_class
USE Model_class
USE Transform_class, ONLY : TBS, RSI

! Modulo per il calcolo del vettore degli scarti R
!
! PUBLIC FUNCTIONS:
! R(M, N, X, F)

IMPLICIT NONE

PRIVATE :: uv

CONTAINS

! ============================================================
SUBROUTINE R(M, N, X, F)

! Calcolo del vettore dei parametri R
!
! PARAMETRI DI INPUT
! M : numero di funzioni (lunghezza di F)
! N : lunghezza del vettore di stato
! X : vettore di stato
!
! PARAMETRI DI OUTPUT:
! F : Valori delle funzioni in X

IMPLICIT NONE

! Variables
INTEGER, INTENT(IN) :: M, N
REAL(8), INTENT(IN) :: X(N)
REAL(8), INTENT(OUT) :: F(M)
INTEGER :: q
REAL(8) :: alpha, delta, deltaS1, t, t0, g_i1_S(4), g_i2_S(4), year0

! Inizio procedimento
CALL mjd2dat(Epoch(1), year0, t0)
DO q = 1, M
	CALL AssignP(X)

	CALL GetVectS(q, t0, alpha, delta, deltaS1, g_i1_S, g_i2_S, t)
	
	F(q) =  w(t, g_i1_S(1:3), g_i2_S(1:3), Res(q, 3))
END DO

END SUBROUTINE R
! ============================================================

! ============================================================
REAL(8) FUNCTION w(t, g_i1_S, g_i2_S, resid)

! Calcolo di w
!
! PARAMETRI DI INPUT
! t      : tempo [sec]
! g_i1_S :
! g_i2_S :
! resid  :
!
! PARAMETRI DI OUTPUT:
! w      : 

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: t, g_i1_S(3), g_i2_S(3), resid
real(8) :: u, v, model, m

! Inizio procedimento
u = uv(t, g_i1_S)
v = uv(t, g_i2_S)
model = u-v
m = lambda1*resid
w = model-m

IF ((tunnel.EQ..FALSE.).AND.(brute.EQ..FALSE.)) THEN
	WRITE (7,*) model, m, w
END IF

END FUNCTION w
! ============================================================

! ============================================================
REAL(8) FUNCTION uv(t, g_i_S)

! Calcolo di u o di v, proiezioni del vettore congiungente il CM e il CF della Soyuz f_I rispettivamente sul
! versore che dal CM della Spyuz punta al primo satellite GPS g_i1_I e sul versore che dal CM della Soyuz
! punta al secondo satellite GPS g_i2_I, sempre restando nel sdr inerziale equatoriale
!
! PARAMETRI DI INPUT
! t      : tempo [sec]
! X      : vettore di stato contenette: A, omega, phi, dtheta_0, theta_0,
!          dlambda_0, lambda_0, dpsi_0, psi_0, f_b_1, f_b_2, f_b_3 
! g_i_S  : versore che dal CM della Soyuz punta a uno dei satelliti GPS coinvolti nel calcolo delle doppie differenze
!
! PARAMETRI DI OUTPUT:
! uv      : proiezione di f_S su g_i_S

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: t, g_i_S(3)
REAL(8) :: f_S(3), f_B(3), lambda, theta, psi

! Inizio procedimento
CALL Assignament(t, f_B, lambda, theta, psi)

CALL TBS(lambda, theta, psi, f_B, f_S)

uv = DOT_PRODUCT(g_i_S, f_S)

END FUNCTION uv
! ============================================================

! ============================================================
SUBROUTINE Assignament(t, f_B, lambda, theta, psi)

! Assegnai valori contenuti nel vettore di stato X alle specifiche variabili
!
! PARAMETRI DI INPUT:
! t         : tempo [sec]
!
! PARAMETRI DI OUTPUT:
! f_B       :
! lambda    :
! theta     :
! psi       :

! Variables
REAL(8), INTENT(IN) :: t
REAL(8), INTENT(OUT) :: f_B(3), lambda, theta, psi

! Inizio procedimento
f_B = (/ fb1, fb2, fb3 /)
lambda = lambda_calc(A, omega, t, phi, dl0, l0)
theta = theta_calc(t, dth0, th0)
psi = psi_calc(t, dps0, ps0)

END SUBROUTINE Assignament
! ============================================================

! ============================================================
SUBROUTINE GetVectS(q, t0, alpha, delta, deltaS1, g_i1_S, g_i2_S, t)

! 

IMPLICIT NONE 

! Variables
INTEGER, INTENT(IN) :: q
REAL(8), INTENT(IN) :: t0
REAL(8), INTENT(OUT) :: alpha, delta, deltaS1, g_i1_S(3), g_i2_S(3), t
REAL(8) :: s_2(3), year, g_i1_I(3), g_i2_I(3), epsilon

! Inizio procedimento
CALL mjd2dat(Epoch(q), year, t)

s_2 = s2(Sun(q, 1:3), Soy(q, 1:3))

alpha = AR(s_2)
delta = Dec(s_2)
IF (t.EQ.t0) THEN
	IF (delta.EQ.0) THEN
		WRITE(*,*) 'Error: zero Sun declination! Select different inital date.'
		STOP
	END IF
	epsilon = Eps(alpha, delta)
END IF

deltaS1 = Decli(alpha, epsilon)

g_i1_I = giI(GPS1(q, 2:4), Soy(q, 1:3))
g_i2_I = giI(GPS2(q, 2:4), Soy(q, 1:3))

g_i1_S(1:3) = MATMUL(TRANSPOSE(RSI(alpha, delta, deltaS1)), g_i1_I)
g_i2_S(1:3) = MATMUL(TRANSPOSE(RSI(alpha, delta, deltaS1)), g_i2_I)

t = t-t0

END SUBROUTINE GetVectS
! ============================================================

END MODULE ParamS_class