MODULE RotoTrasl_class

USE Globals_class, ONLY : deg2rad, identity_matrix

! Modulo che contiene le funzioni di rotazione e rototraslazione
!
! PUBLIC FUNCTIONS:
! rot(a, b, g)
! trasm(Rij, Pij)

IMPLICIT NONE

CONTAINS

! ============================================================
FUNCTION rot(a, b, g)

! Operatore di rotazione secondo la sequanza di rotazioni:
! (alpha, z), (beta, y), (gamma, x)
!
! PARAMETRI DI INPUT:
! a : alpha [deg]
! b : beta [deg]
! g : gamma [deg]
!
! PARAMETRI DI OUTPUT:
! rot : operatore di roto-traslazione per la sequenza indicata

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: a, b, g
REAL(8) :: rot(3,3), ar, br, gr, sa, sb, sg, ca, cb, cg

! Inizio procedimento
ar = deg2rad(a)
br = deg2rad(b)
gr = deg2rad(g)

sa = DSIN(ar)
sb = DSIN(br)
sg = DSIN(gr)

ca = DCOS(ar)
cb = DCOS(br)
cg = DCOS(gr)

rot(1,1) = ca*cb
rot(1,2) = ca*sb*sg-sa*cg
rot(1,3) = ca*sb*cg+sa*sg
rot(2,1) = sa*cb
rot(2,2) = sa*sb*sg+ca*cg
rot(2,3) = sa*sb*cg-ca*sg
rot(3,1) = -sb
rot(3,2) = cb*sg
rot(3,3) = cb*cg

END FUNCTION rot
! ============================================================

! ============================================================
FUNCTION trasm(Rij, Pij)

! Costruzione dell'operatore di roto-traslazione
!
! PARAMETRI DI INPUT:
! Rij   : matrice di rotazione
! Pij   : origine del sdr i espressa nel sdr j
!
! PARAMETRI DI OUTPUT:
! trasm : operatore di roto-traslazione

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: Rij(3,3), Pij(3)
REAL(8) :: trasm(4,4)

! Inizio procedimento
trasm = identity_matrix(4)
trasm(1:3, 1:3) = Rij
trasm(1:3, 4) = Pij

END FUNCTION trasm
! ============================================================

END MODULE RotoTrasl_class