! ============================================================
SUBROUTINE Tunneling(XLM, m, n, XFin)

USE Numerical_libraries
USE Globals_class, ONLY : dimX, dimRed, Xs, lam, lamm, minsloc, count, active, tunnel, XLB, XUB
USE Param_class, ONLY : R
USE Verbatim_class, ONLY : Verba

! Subroutine per l'applicazione del medodo di Tunneling

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: XLM(dimX)
INTEGER, INTENT(IN) :: m, n
REAL(8), INTENT(OUT) :: XFin(dimX)
INTEGER :: i, j, IPARAM(6)
REAL(8) :: T, Rx(dimRed), F, XT(dimX), min2(dimX+1)
LOGICAL :: mode
REAL(8) :: XSCALE(dimX), FSCALE, RPARAM(7), XTT(dimX), X0(dimX), XLMT(dimX)
EXTERNAL :: TCmin, TEmin

! Inizio procedimento
mode = .FALSE.
X0 = XLM

DO i = 1, m
	PRINT *, 'Loc', i
	tunnel = .TRUE.
	count = 0
	! Fase di ricerca di minimo locale
!	CALL LevenMarq(X0, XFin, IPARAM)
	CALL LevenMarqB(X0, XLMT, IPARAM)
	tunnel = .TRUE.
	CALL Verba(XLMT, IPARAM)
	CALL R(dimRed, dimX, XLMT, Rx)
	F = 0.5*DOT_PRODUCT(Rx,Rx)
	WRITE(8, *)
	WRITE(8, *) 'XLMT: ', XLMT, F
	IF (i>1) THEN
		IF (F.GE.min2(dimX+1)) THEN
			WRITE(*,*) 'Warning!!! Function value incresed during iterations!!!'
			EXIT
		ELSE
			min2 = (/ XLMT, F /)
		END IF
	ELSE
		min2 = (/ XLMT, F /)
	END IF
	
	! Fase di Tunneling
	lam = 0.7
	T = 0.5
	IF (ALLOCATED(minsloc)) DEALLOCATE(minsloc)
	IF (ALLOCATED(Xs)) DEALLOCATE(Xs)
	IF (ALLOCATED(lamm)) DEALLOCATE(lamm)
	ALLOCATE(minsloc(dimX+1, m))
	ALLOCATE(Xs(dimX+1))
	ALLOCATE(lamm(m))
	Xs = (/ XLMT, F /)

	DO j = 1, n
		PRINT *, 'Tun', j
		IF (mode.EQ..TRUE.) THEN
			CALL Initial(XLMT, T, mode, XT)
		ELSE
			CALL Initial(XLMT, lam, mode, XT)
		END IF

		! Trovo il minimo di T
		XSCALE = 1.d0
		FSCALE = 1.d0
		IPARAM(1) = 0
		
		IF (mode.EQ..TRUE.) THEN
!			CALL DUMINF(TCmin, dimX, XT, XSCALE, FSCALE, IPARAM, RPARAM, X, T)
			CALL DBCONF(TCmin, dimX, XT, 0, XLB, XUB, XSCALE, FSCALE, IPARAM, RPARAM, XTT, T)
		ELSE
!			CALL DUMINF(TEmin, dimX, XT, XSCALE, FSCALE, IPARAM, RPARAM, X, T)
			CALL DBCONF(TCmin, dimX, XT, 0, XLB, XUB, XSCALE, FSCALE, IPARAM, RPARAM, XTT, T)
		END IF

		! Verifico se la funzione nel minimo  inferiore a zero
		IF (T<0) THEN
			X0 = XTT
			CALL R(dimRed, dimX, XTT, Rx)
			F = 0.5*DOT_PRODUCT(Rx,Rx)
			WRITE(8, *)
			WRITE(8, *) 'XTT: ', XTT, F
			EXIT
		ELSE
			count = count+1
			minsloc(:,count) = (/ XTT, T /)
		END IF
	END DO

	IF (j.EQ.n) THEN
		WRITE(*,*) 'Uscita dal tunnel! ', i, j
		EXIT
	END IF
END DO

XFin = min2(1:dimX)

END SUBROUTINE Tunneling
! ============================================================

! ============================================================
SUBROUTINE Initial(Xstar, coef, mode, XT)

USE Numerical_libraries
USE Globals_class, ONLY : dimX

!

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: Xstar(dimX), coef
LOGICAL, INTENT(IN) :: mode
REAL(8), INTENT(OUT) :: XT(dimX)
INTEGER :: ISEED, i
REAL(8) :: eps, r(dimX), sign

! Inizio procedimento
ISEED = 1234567
CALL RNSET(ISEED)
CALL RNOPT(7)

IF (mode.EQ..TRUE.) THEN
	IF (coef<0.1d0) THEN
		eps = 0.1d0*ABS(coef)
	ELSE
		eps = 0.1d0
	END IF	
ELSE
	eps = coef/LOG(100.d0)
END IF

DO i = 1, dimX
	sign = DRNUNF()
	r(i) = DRNUNF()
	IF (sign<0.5d0) THEN
		r(i) = -r(i)
	END IF
END DO

XT = Xstar+eps*r

END SUBROUTINE Initial
! ============================================================

! ============================================================
SUBROUTINE TCmin(N, X, T)

!

IMPLICIT NONE

! Variables
INTEGER, INTENT(IN) :: N
REAL(8), INTENT(IN) :: X(N)
REAL(8), INTENT(OUT) :: T
EXTERNAL :: TCmobil

! Inizio procedimento
CALL TCmobil(X, T)

END SUBROUTINE TCmin
! ============================================================

! ============================================================
SUBROUTINE TCmobil(X, T)

USE Globals_class, ONLY : dimX, norm2, lamm, minsloc, count

!

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: X(dimX)
REAL(8), INTENT(OUT) :: T
INTEGER :: p

! Inizio procedimento
CALL TC(X, T)
DO p = 1, count
	T = T/(norm2(X-minsloc(1:dimX,count)))**(2*lamm(p))
END DO

END SUBROUTINE TCmobil
! ============================================================

! ============================================================
SUBROUTINE TC(X, T)

USE Globals_class, ONLY : dimX, dimRed, norm2, Xs, lam
USE Param_class, ONLY : R

!

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: X(dimX)
REAL(8), INTENT(OUT) :: T
REAL(8) :: Rx(dimRed), Rxs(dimRed), F, FS

! Inizio procedimento
CALL R(dimRed, dimX, X, Rx)
F = 0.5d0*DOT_PRODUCT(Rx, Rx)
CALL R(dimRed, dimX, Xs, Rxs)
Fs = 0.5d0*DOT_PRODUCT(Rxs, Rxs)

T = (F-Fs)/((norm2(X-Xs))**(2*lam))

END SUBROUTINE TC
! ============================================================

! ============================================================
SUBROUTINE TEmin(N, X, T)

!

IMPLICIT NONE

! Variables
INTEGER, INTENT(IN) :: N
REAL(8), INTENT(IN) :: X(N)
REAL(8), INTENT(OUT) :: T
EXTERNAL :: TEmobil

! Inizio procedimento
CALL TEmobil(X, T)

END SUBROUTINE TEmin
! ============================================================

! ============================================================
SUBROUTINE TEmobil(X, T)

USE Globals_class, ONLY : dimX, norm2, lamm, minsloc, count

!

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: X(dimX)
REAL(8), INTENT(OUT) :: T
INTEGER :: p

! Inizio procedimento
CALL TC(X, T)
DO p = 1, count
	T = T*EXP(lamm(p)/(norm2(X-minsloc(1:dimX,count)))**2)
END DO

END SUBROUTINE TEmobil
! ============================================================

! ============================================================
REAL(8) FUNCTION TE(X)

USE Globals_class, ONLY : dimX, dimRed, norm2, Xs, lam
USE Param_class, ONLY : R

!

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: X(dimX)
REAL(8) :: Rx(dimRed), Rxs(dimRed), F, FS

! Inizio procedimento
CALL R(dimRed, dimX, X, Rx)
F = 0.5d0*DOT_PRODUCT(Rx, Rx)
CALL R(dimRed, dimX, Xs, Rxs)
Fs = 0.5d0*DOT_PRODUCT(Rxs, Rxs)

TE = (F-Fs)*EXP(lam/(norm2(X-Xs))**2)

END FUNCTION TE
! ============================================================