!****h* Attitude/Vectors_class
! NAME
! Vectors_class -- Calculates versors s_2, g_i1_I and g_i2_I
! FUNCTION
! Contains functions to calculate the versors s_2, g_i1_I and g_i2_I
! in the inertial equatorial coordinate system
! AUTHOR
! Paolo Zoccarato
! CREATION DATE
! 2006/06/20
! MODIFICATION HISTORY
! +------------------------------------------------------------------+
! |  name      |  date  |                  reason                    |
! |------------|--------|--------------------------------------------|
! |            |        |  No one at the moment                      |
! +------------|--------|--------------------------------------------+
! |            |        |                                            |
! +------------------------------------------------------------------+
! COMMANDS
! PUBLIC FUNCTIONS:
!	s2(Sun, Soy) -- Calculates versor s_2, that indicates the direction from the Soyuz CM to the Sun
!		INPUT PARAMETERS:
!			Sun    : SunStat type object that contains the Sun position at the considerated instant
!			Soy    : SoyStat type object that contains the Soyuz position at the considerated instant
!		OUTPUT PARAMETERS:
!			s_2(3) : Versor from Soyuz CM to Sun
!
!	giI(GPS, Soy) -- Calculates 
!		INPUT PARAMETERS:
!			Sun    : SunStat type object that contains the Sun position at the considerated instant
!			GPS    : GPSStat type object that contains the GPS satellita position at the considerated instant
!		OUTPUT PARAMETERS:
!			Dec     : Right Ascension of the vector "vect" in the inertial equatorial coordinate system
! USES
! This module uses the parameters "i, j, k, norm2" of the module "Globals_class"
! NOTES
! Develloped with Compaq Visual Fortran for Windows on Windows XP
! SOURCE

MODULE Vectors_class

USE Globals_class, ONLY : norm2

! Modulo che
!
! PUBLIC FUNCTIONS:

IMPLICIT NONE

CONTAINS

! ============================================================
FUNCTION s2(posSun, posSoy)

! Funzione che calcola il versore s2 nel sdr inerziale equatoriale che indica la direzione
! dal CM della Soyuz al Sole
!
! PARAMETRI DI INPUT
! posSun(3) :
! posSoy(3) :
!
! PARAMETRI DI OUTPUT:
! s2        : versore che dal CM della Soyuz punta al Sole

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: posSun(3), posSoy(3)
REAL(8) :: s2(3)

! Inizio procedimentoo 
s2 = (posSun - posSoy)/norm2(posSun - posSoy)

END FUNCTION 
! ============================================================

! ============================================================
FUNCTION giI(posGPS, posSoy)

! Funzione che calcola il versore g_i_I nel sdr inerziale equatoriale che indica la direzione
! dal CM della Soyuz a uno dei satelliti GPS utilizzati per il calcolo delle doppie differenze
!
! PARAMETRI DI INPUT
! posGPS(3) :
! posSoy(3) :
!
! PARAMETRI DI OUTPUT:
! giI       : versore che dal CM della Soyuz punta al satellite GPS

IMPLICIT NONE

! Variables
REAL(8), INTENT(IN) :: posGPS(3), posSoy(3)
REAL(8) :: giI(3)

! Inizio procedimento
giI = (posGPS - posSoy)/norm2(posGPS - posSoy)

END FUNCTION 
! ============================================================

END MODULE Vectors_class
!*****